/* TroiaPreview
 * version 0.4.1 date 2005-02-06
 * Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <string.h>

#include "Tools.h"
#include "Globals.h"
#include "Pixel.h"
#include "Text.h"
#include "Ticker.h"

//private constants
#define TickerTextSize 256

//type for a ticker
typedef struct _tTicker
{
  int Active; //if ticker is active
  char Text[TickerTextSize]; //the text to show
  tPixel3D TopLeft; //starting point of ticker
  unsigned char Dir; //direction of ticker
  unsigned int Width; //width of ticker in pixels
  unsigned short ForeColor, BackColor; //color of text
  double RefTime; //reference time (when to start scrolling text)
  double Speed; //speed of ticker in pixels per second
} tTicker;

//list of tickers
tTicker Tickers[TickerCnt];

//initialize tickers
void TickerInit( )
{
  int I;

  //disable all tickers
  for( I = 0; I < TickerCnt; I++ )
  {
    Tickers[I].Active = 0; //not active
    Tickers[I].Text[0] = 0; //no text
    Tickers[I].TopLeft.Wall = 0; //invalid pixel
    Tickers[I].Dir = 0; //invalid direction
    Tickers[I].Width = TickerMinWidth; //minimal size
    Tickers[I].ForeColor = 0xFF; //white on black
    Tickers[I].BackColor = 0x00;
    Tickers[I].RefTime = 0.0; //some RefTime
    Tickers[I].Speed = TickerMinSpeed; //very slow
  }
}

//check if ticker is active
int TickerGetActive( unsigned int TickerNo )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return 0;
  //return if ticker active
  return Tickers[TickerNo].Active;
}

//enable / disable ticker
void TickerSetActive( unsigned int TickerNo, int Active )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //save if ticker is active now
  Tickers[TickerNo].Active = Active;
}

//get ticker text length
unsigned int TickerGetTextLen( unsigned int TickerNo )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return 0;
  //return ticker text length
  return strlen( Tickers[TickerNo].Text );
}

//set ticker text
void TickerSetText( unsigned int TickerNo, char * pTxt )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //save new text
  strncpy( Tickers[TickerNo].Text, pTxt, sizeof( Tickers[TickerNo].Text ) );
  Tickers[TickerNo].Text[sizeof( Tickers[TickerNo].Text ) - 1] = 0;
}

//get ticker position, direction and width
void TickerGetPos( unsigned int TickerNo, tPixel3D * pPix, unsigned char * pDir, unsigned int * pWidth )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //return position, direction and width
  *pPix = Tickers[TickerNo].TopLeft;
  *pDir = Tickers[TickerNo].Dir;
  *pWidth = Tickers[TickerNo].Width;
}

//set ticker position, direction and width
void TickerSetPos( unsigned int TickerNo, tPixel3D Pix, unsigned char Dir, unsigned int Width )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //save new position, direction and width
  Tickers[TickerNo].TopLeft = Pix;
  Tickers[TickerNo].Dir = Dir;
  Tickers[TickerNo].Width = LongMinMax( Width, TickerMinWidth, TickerMaxWidth );
}

//get ticker color
void TickerGetColor( unsigned int TickerNo, unsigned short * pForeColor, unsigned short * pBackColor )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //return color
  *pForeColor = Tickers[TickerNo].ForeColor;
  *pBackColor = Tickers[TickerNo].BackColor;
}

//set ticker color
void TickerSetColor( unsigned int TickerNo, unsigned short ForeColor, unsigned short BackColor )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //save new ticker color
  Tickers[TickerNo].ForeColor = ShortMinMax( ForeColor, 0, 256 ); //256 is transparent
  Tickers[TickerNo].BackColor = ShortMinMax( BackColor, 0, 256 ); //256 is transparent
}

//get ticker reference time
double TickerGetRefTime( unsigned int TickerNo )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return 0.0;
  //return ticker reference time
  return Tickers[TickerNo].RefTime;
}

//set ticker reference time
void TickerSetRefTime( unsigned int TickerNo, double RefTime )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //save new ticker reference time
  Tickers[TickerNo].RefTime = DoubleMinMax( RefTime, -TimeRange, TimeRange );
}

//get ticker speed
double TickerGetSpeed( unsigned int TickerNo )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return 0.0;
  //return ticker speed
  return Tickers[TickerNo].Speed;
}

//set ticker speed
void TickerSetSpeed( unsigned int TickerNo, double Speed )
{
  //check ticker number
  if( TickerNo >= TickerCnt )
    return;
  //save new ticker speed
  Tickers[TickerNo].Speed = DoubleMinMax( Speed, TickerMinSpeed, TickerMaxSpeed );
}

//draw a ticker
static void TickerDraw( tTicker * pTicker, double CurrentTime )
{
  unsigned int ScrollTotal, ScrollNow, Pad, Skip;

  //do nothing if ticker inactive
  if( ! pTicker->Active )
    return;

  //get number of pixels to scroll
  ScrollTotal = pTicker->Width + TextWidth( pTicker->Text );
  //get number of pixels the text must be scrolled at the moment
  ScrollNow = (unsigned int)DoubleModulo( pTicker->Speed * (CurrentTime - pTicker->RefTime) + 0.5, ScrollTotal );

  //scrolling text into view
  if( ScrollNow < pTicker->Width )
  {
    Pad = pTicker->Width - ScrollNow;
    Skip = 0;
  }
  //scrolling text though view and out of view
  else
  {
    Pad = 0;
    Skip = ScrollNow - pTicker->Width;
  }

  //draw text in temporary frame
  TextOutputEx( pTicker->Text, pTicker->TopLeft, pTicker->Dir,
          pTicker->ForeColor, pTicker->BackColor, 0,
          Pad, Skip, pTicker->Width );
}

//called periodically to do some work
void TickerTask( )
{
  double CurrentTime;
  int I;

  //get time
  CurrentTime = TimeGet( );

  //draw all active tickers
  for( I = 0; I < TickerCnt; I++ )
    TickerDraw( &Tickers[I], CurrentTime );
}
