/* TroiaPreview
 * version 0.4.1 date 2005-02-06
 * Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <stdlib.h>

#include "Tools.h"
#include "Globals.h"
#include "Pixel.h"
#include "Frame.h"
#include "Worms.h"

//private global variables
unsigned int WormCount = 0; //number of worms
double WormInterval = 0.0; //interval between steps (worms stopped)
double WormLastStepTime; //time of last step
tPixel3D WormPixels[WormMaxCount]; //worms
unsigned char WormDirs[WormMaxCount]; //directions of worms
unsigned char WormColors[WormMaxCount]; //colors of worms

//do a worm step
static void WormStep( )
{
  unsigned int I;

  //process worms
  for( I = 0; I < WormCount; I++ )
  {
    //probably change direction
    if( rand( ) < RAND_MAX >> 6 )
      WormDirs[I] = DirRotate( WormDirs[I], ((rand( ) & 1) << 1) - 1 );

    //probably change color
    if( rand( ) < RAND_MAX >> 6 )
      WormColors[I] = (unsigned char)ShortMinMax( (short)WormColors[I] + rand( ) % 65 - 32, 0, 255 );

    //move
    Pixel3DMove( WormPixels[I], WormDirs[I], &WormPixels[I], &WormDirs[I] );
    if( WormPixels[I].Wall == 0 ) //invalid pixel (e.g. lost in corner)
    {
      WormPixels[I] = PixelRandom( ); //create new pixel
      WormDirs[I] = DirRandom( );
    }

    //draw pixel
    FrameSetPixel( WormPixels[I], WormColors[I] );
  }
}

//start worms
void WormStart( unsigned int Size, double Interval )
{
  unsigned int I;

  WormCount = LongMinMax( Size, WormMinCount, WormMaxCount );
  WormInterval = DoubleMinMax( Interval, WormMinInterval, WormMaxInterval );
  WormLastStepTime = TimeGet( ) - WormInterval;

  //initialize worms
  for( I = 0; I < WormCount; I++ )
  {
    WormPixels[I] = PixelRandom( );
    WormDirs[I] = DirRandom( );
    WormColors[I] = (unsigned char)rand( );
  }
}

//stop worms
void WormStop( )
{
  WormInterval = 0.0;
}

//called periodically to do some work
void WormTask( )
{
  double CurrentTime;

  //exit if worms not running
  if( WormInterval < WormMinInterval * 0.5 ) //use * 0.5 to get a safe decision threshold
    return;

  //get time
  CurrentTime = TimeGet( );

  //exit if interval not elapsed
  if( CurrentTime < WormInterval + WormLastStepTime )
    return;

  //do a step
  WormStep( );

  //restart interval
  WormLastStepTime = CurrentTime;
}

